#include <avr/interrupt.h>

#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>
#include "nokia5110.h"
#include "nokia5110_chars.h"

#define OCTAVE_SIZE 13
#define ON 1
#define OFF 0
int frecv_note[OCTAVE_SIZE] = {262, 277, 294, 311, 330, 349, 370, 392, 415, 440, 466, 494, 523};
int apasat[OCTAVE_SIZE] = {OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF};

#define INT_MIC 0
#define INT_MARE 1
#define NORMAL 0
#define RECORD 1
#define PLAY 2
#define TERTA 1
#define MORSE_LUNG		150
#define MORSE_SCURT		50
#define MORSE_PAUZA		50

uint8_t interval_state = INT_MARE;
uint8_t running_state = NORMAL;
uint8_t interval_type = NORMAL;
uint8_t current_note;

ISR(PCINT2_vect)
{
    if ((PINC & (1 << PC0)) == 0) {
        if(running_state == NORMAL)
            running_state = RECORD;
        else if(running_state == RECORD)
            running_state = PLAY;
        else if(running_state == PLAY)
            running_state = NORMAL;
    } else if ((PINC & (1 << PC1)) == 0) {
        if(interval_type == NORMAL)
            interval_type = TERTA;
        else    interval_type = NORMAL;
    }else if ((PINC & (1 << PC7)) == 0) {
        if(interval_state == INT_MARE)
            interval_state = INT_MIC;
        else interval_state = INT_MARE;
    }
}


/* initializeaza intrarile si iesirile */
void IO_init()
{

    /*Setare pini PD0, PB6, PB7, PC1 ca iesire pentru led-uri*/ 
    DDRD |= (1 << PD0);
    DDRB |= (1 << PB6) | (1 << PB7);
    DDRC |= (1 << PC1);
    PORTD &= ~(1 << PD0);
    PORTB &= ~(1 << PB6) & ~(1 << PB7);
    PORTC &= ~(1 << PC1);

    /*Setare PD4 ca pin de iesire pentru buzzer*/
    DDRD |= (1 << PD4);
	/*Oprire buzzer*/
	PORTD &= ~(1 << PD4);

    /*Setare pini PA0->PA7 si PB0, PB1, PB3, PB4, PB5 ca intrare pentru butoanele de claviatura,
    iar butoanele PD1, PD5, PD6,  ca intrare pentru butoanele de control si activare
    rezistente de pull-up*/
    DDRA = 0; 
    PORTA = 0b11111111;

    DDRB &= ~(1 << PB0) & ~(1 << PB1) & ~(1 << PB3) & ~(1 << PB4) & ~(1 << PB5);
    PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB3) | (1 << PB4) | (1 << PB5);

    DDRD &= ~(1 << PD1) & ~(1 << PD5) & ~(1 << PD6);
    PORTD |= (1 << PD1) | (1 << PD5) | (1 << PD6);

    /* Activare PIN CHANGE INTERRUPT ENABLE pentru piniit PCINT23-0*/
    //PCICR |= (1 << PCIE0) | (1 << PCIE1) | (1 << PCIE3);
    
    /* Activare intrerupere pentru toti pinii butoanelor PA0->PA7, PB0, PB1,
    PB3, PB4, PB5, PD1, PD5, PD6*/
    //PCMSK0 = 0b11111111;
    //PCMSK1 |= (1 << PCINT8) | (1 << PCINT9) | (1 << PCINT11) | (1 << PCINT12) | (1 << PCINT13);
    //PCMSK3 |= (1 << PCINT25) | (1 << PCINT29) | (1 << PCINT30);
}

void speaker_reda(int nota) {
	int perioada_nota = 1000000l / frecv_note[nota];
	int i;

	/*
	 * Nu putem folosi variable ca argument pentru functia
	 * _delay_us(), asa ca folosim acest "workaround".
	 */
	PORTD |= (1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);

	PORTD &= ~(1 << PD4);
	for (i = 0; i < perioada_nota / 2; ++i)
		_delay_us(1);
}

void pian() {
	int current_note = 0;
    while(1) {
    	if((PINA & (1 << PA0)) == 0) {
    		if(apasat[0] == OFF) {
    			apasat[0] = ON;
    			current_note = 0;
    			_delay_ms(10);
    		}
    	} else apasat[0] = OFF;

    	if((PINA & (1 << PA1)) == 0) {
    		if(apasat[1] == OFF) {
    			apasat[1] = ON;
    			current_note = 1;
    			_delay_ms(10);
    		}
    	} else apasat[1] = OFF;

    	if((PINA & (1 << PA2)) == 0) {
    		if(apasat[2] == OFF) {
    			apasat[2] = ON;
    			current_note = 2;
    			_delay_ms(10);
    		}
    	} else apasat[2] = OFF;

    	if((PINA & (1 << PA3)) == 0) {
    		if(apasat[3] == OFF) {
    			apasat[3] = ON;
    			current_note = 3;
    			_delay_ms(10);
    		}
    	} else apasat[3] = OFF;

    	if((PINA & (1 << PA4)) == 0) {
    		if(apasat[4] == OFF) {
    			apasat[4] = ON;
    			current_note = 4;
    			_delay_ms(10);
    		}
    	} else apasat[4] = OFF;

    	if((PINA & (1 << PA5)) == 0) {
    		if(apasat[5] == OFF) {
    			apasat[5] = ON;
    			current_note = 5;
    			_delay_ms(10);
    		}
    	} else apasat[5] = OFF;

    	if((PINA & (1 << PA6)) == 0) {
    		if(apasat[6] == OFF) {
    			apasat[6] = ON;
    			current_note = 6;
    			_delay_ms(10);
    		}
    	} else apasat[6] = OFF;

    	if((PINA & (1 << PA7)) == 0) {
    		if(apasat[7] == OFF) {
    			apasat[7] = ON;
    			current_note = 7;
    			_delay_ms(10);
    		}
    	} else apasat[7] = OFF;

    	if((PINB & (1 << PB0)) == 0) {
    		if(apasat[8] == OFF) {
    			apasat[8] = ON;
    			current_note = 8;
    			_delay_ms(10);
    		}
    	} else apasat[8] = OFF;

    	if((PINB & (1 << PB1)) == 0) {
    		if(apasat[9] == OFF) {
    			apasat[9] = ON;
    			current_note = 9;
    			_delay_ms(10);
    		}
    	} else apasat[9] = OFF;

    	if((PINB & (1 << PB3)) == 0) {
    		if(apasat[10] == OFF) {
    			apasat[10] = ON;
    			current_note = 10;
    			_delay_ms(10);
    		}
    	} else apasat[10] = OFF;

    	if((PINB & (1 << PB4)) == 0) {
    		if(apasat[11] == OFF) {
    			apasat[11] = ON;
    			current_note = 11;
    			_delay_ms(10);
    		}
    	} else apasat[11] = OFF;

    	if((PINB & (1 << PB5)) == 0) {
    		if(apasat[12] == OFF) {
    			apasat[12] = ON;
    			current_note = 12;
    			_delay_ms(10);
    		}
    	} else apasat[12] = OFF;

    	int i, state = 0;
    	for(i = 0; i < OCTAVE_SIZE; i++)
    		if(apasat[i] == ON)
    			state = 1;
    	if(state == 1)
    		speaker_reda(current_note);
    }
}


void simplu() {
	while(1) {
		if((PIND & (1 << PD1)) == 0)
			PORTD |= (1 << PD1);
		else if((PIND & (1 << PD1)) == 1)
			PORTD &= ~(1 << PD1);
		_delay_ms(200);
	}
}

/* Genereaza sunetul pentru tipul de ton folosit: lung (150ms) sau scurt (50ms). */
void speaker_morse(int tip_morse) {
	int i;

	/* Alternam starea pinului PD4 la fiecare milisecunda. */
	for (i = 0; i < tip_morse; ++i) {
		PIND = (1 << PD4);

		_delay_ms(1);
	}
}


void test_all(void) {

	/* Opresc speaker-ul. */
	PORTD &= ~(1 << PD4);

	while (1) {
		if ((PINA & (1 << PA0)) == 0 || (PINA & (1 << PA1)) == 0 || (PINA & (1 << PA2)) == 0 || (PINA & (1 << PA3)) == 0 ||
			(PINA & (1 << PA4)) == 0 || (PINA & (1 << PA5)) == 0 || (PINA & (1 << PA6)) == 0 || (PINA & (1 << PA7)) == 0 ||
			(PIND & (1 << PD1)) == 0 || (PIND & (1 << PD5)) == 0 || (PIND & (1 << PD6)) == 0) {
			PORTB |= (1 << PB7);
			PORTB |= (1 << PB6);
			PORTC |= (1 << PC1);
			PORTD |= (1 << PD0);

			speaker_morse(MORSE_SCURT);
		}
		else if (((PINB & (1 << PB0)) == 0) || ((PINB & (1 << PB1)) == 0) || ((PINB & (1 << PB3)) == 0) || 
			((PINB & (1 << PB4)) == 0) || ((PINB & (1 << PB5)) == 0)) {
			PORTB &= ~(1 << PB7);
			PORTB &= ~(1 << PB6);
			PORTC &= ~(1 << PC1);
			PORTD &= ~(1 << PD0);			
			speaker_morse(MORSE_LUNG);
		}
		PORTD &= ~(1 << PD4);
		_delay_ms(50);
	}
}

int main(void)
{
    // initializari si testare interfeta afisare/debug

    _delay_ms(1000);

    // codul relevant laboratorului incepe aici
    //sei();

    IO_init();

    nokia_lcd_init();
    nokia_lcd_power(1);
    nokia_lcd_clear();
    nokia_lcd_set_cursor(10, 10);
    nokia_lcd_write_string("Hello", 1);
    nokia_lcd_render();
    pian();
    //test_all();

    return 0;
}
